;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; PROJECT: Carnegie Mellon Planetary Rover Project
;;;          Task Control Architecture 
;;; 
;;; MODULE: tca - lisp
;;;
;;; FILE: tca-compile.lisp
;;;
;;; ABSTRACT:
;;; 
;;; Task Control Architecture - Lisp Compile File
;;;
;;; $Revision: 1.2 $
;;; $Date: 1995/07/08 18:24:06 $
;;; $Author: rich $
;;;
;;; REVISION HISTORY
;;;
;;;  $Log: tca-compile.lisp,v $
;;; Revision 1.2  1995/07/08  18:24:06  rich
;;; Change all /afs/cs to /afs/cs.cmu.edu to get ride of conflict problems.
;;;
;;; Revision 1.1  1994/03/27  22:48:55  rich
;;; Loads are not done relative to the tca.lisp file and not with a
;;; hard coded pathname.
;;; The tca-compile file can be used to compile the list files.  Just load it.
;;;
;;;

;;; Get the tca root directory.

(defparameter *tcaDir* 
  "/afs/cs.cmu.edu/project/TCA/release/omega"
  "The tca root directory")

(eval-when (eval load compile) 
  (setf *tcaDir* (make-pathname :directory 
				(reverse 
				 (rest
				  (reverse
				   (pathname-directory *load-truename*))))))
  (format t "Loading from directory ~A" *tcaDir*)
  )

;;; Load the lisp version of the code.
(load (merge-pathnames (make-pathname :directory "lisp" :name "tca.lisp")
		       *tcaDir*))

;;; compile the relevant files.
(compile-file (merge-pathnames (make-pathname :directory "lisp" :name "tca") 
			       *tcaDir*))
(compile-file (merge-pathnames (make-pathname :directory "lisp" 
					      :name "formatters") 
			       *tcaDir*))
(compile-file (merge-pathnames (make-pathname :directory "lisp" :name "map") 
			       *tcaDir*))
(compile-file (merge-pathnames (make-pathname :directory "lisp" 
					      :name "primFmttrs") 
			       *tcaDir*))
(compile-file (merge-pathnames (make-pathname :directory "lisp" 
					      :name "tcaForeignCalls") 
			       *tcaDir*))

;;; Load the compiled version of the code.
(load (merge-pathnames (make-pathname :directory "lisp" :name "tca") *tcaDir*))
 
;;; recompile the relevant files.
(compile-file (merge-pathnames (make-pathname :directory "lisp" :name "tca") 
			       *tcaDir*))
(compile-file (merge-pathnames (make-pathname :directory "lisp" 
					      :name "formatters") 
			       *tcaDir*))
(compile-file (merge-pathnames (make-pathname :directory "lisp" :name "map") 
			       *tcaDir*))
(compile-file (merge-pathnames (make-pathname :directory "lisp" 
					      :name "primFmttrs") 
			       *tcaDir*))
(compile-file (merge-pathnames (make-pathname :directory "lisp" 
					      :name "tcaForeignCalls") 
			       *tcaDir*))
