;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; PROJECT: Carnegie Mellon Planetary Rover Project
;;;          Task Control Architecture 
;;; 
;;; MODULE: tca - lisp
;;;
;;; FILE: tca.lisp
;;;
;;; ABSTRACT:
;;; 
;;; Task Control Architecture - Lisp Load File
;;;
;;; $Revision: 1.12 $
;;; $Date: 1995/08/05 17:20:55 $
;;; $Author: reids $
;;;
;;; REVISION HISTORY
;;;
;;;  $Log: tca.lisp,v $
;;; Revision 1.12  1995/08/05  17:20:55  reids
;;; Brought the LISP version into sync with the C version (pretty much);
;;; Added some more of the "matrix" functions to the LISP side;
;;; Fixed a bug in sending a null string.
;;;
;;; Revision 1.11  1995/07/24  16:29:13  reids
;;; Fix some bugs in STR_Trans, and bring the LISP version up to TCA V8.0
;;;
;;; Revision 1.10  1995/07/08  18:24:07  rich
;;; Change all /afs/cs to /afs/cs.cmu.edu to get ride of conflict problems.
;;;
;;; Revision 1.9  1994/10/13  03:20:43  rich
;;; Changes needed for Allegro V4.2 on sunOS machines.
;;;
;;; Revision 1.8  1994/04/16  19:32:47  rich
;;; Format changes.
;;;
;;; Revision 1.7  1994/03/27  22:48:56  rich
;;; Loads are not done relative to the tca.lisp file and not with a
;;; hard coded pathname.
;;; The tca-compile file can be used to compile the list files.  Just load it.
;;;
;;; Revision 1.6  1994/03/26  21:46:34  rich
;;; Changed a quote to a backquote.
;;;
;;; Revision 1.5  1993/11/21  20:16:41  rich
;;; Added shared library for sun4c_411 sunos machines.
;;; Added install to the makefile.
;;; Fixed problems with global variables.
;;;
;;; Revision 1.4  1993/11/21  20:05:30  rich
;;; test
;;;
;;; Revision 1.3  1993/11/21  20:02:16  rich
;;; Experiment to help load correct versions of code.
;;;
;;; Revision 1.2  1993/06/13  23:26:09  rich
;;; Added comments at the front of each file.
;;; Fixed prefix for log entries.
;;; Made changes needed for vx works style global variables.
;;;

;;; Package information for allegro versions befor 4.0.1
#-(OR CLTL2 CMU)
(in-package "TASK-CONTROL" :nicknames '("TC" "TCA"))

#-(OR CLTL2 CMU) (use-package "TASK-CONTROL" "USER")

#-(OR CLTL2 CMU) 
(export '(;; Communication Return Values
	  Success TimeOut Failure WrongVersion MsgUndefined
	  WrongMsgClass NullReply

	  *WAITFOREVER* defstruct_tca

	  ;; Export constants for message taps.
	  UnknownCondition  WhenSent  BeforeHandling  WhileHandling  
	  AfterHandled  AfterReplied  AfterSuccess  AfterFailure  
	  BeforeAchieving  WhileAchieving 
	  AfterAchieved  WhenAchieved  BeforePlanning  WhilePlanning  
	  AfterPlanned WhenPlanned WhenSuccess WhenFailure  WhenKilled 

	  NO_TPLCONSTR SEQ_ACH SEQ_PLANNING PLAN_FIRST DELAY_PLANNING

	  tcaInform tcaAddConstraint tcaQuery tcaReply tcaNullReply
	  tcaEnableDistributedResponses tcaDisableDistributedResponses
	  tcaQuerySend tcaQueryReceive
	  tcaExpandGoal tcaExpandGoalWithConstraints
	  tcaExecuteCommand tcaExecuteCommandWithConstraints
	  tcaWaitForCommand tcaWaitForCommandWithConstraints
	  tcaExecute tcaExecuteWithConstraints 
	  tcaExecuteCommandAsync tcaExecuteAsync
	  tcaMaybeExecute tcaMaybeExecuteWithConstraints tcaMaybeExecuteAsync
	  tcaSuccess tcaFailure tcaBroadcast tcaMultiQuery tcaMultiReceive
	  tcaServerMachine tcaRootNode tcaDefaultTime tcaGetServer
	  tcaGetConnections tcaWaitUntilReady tcaModuleListen tcaHandleMessage
	  tcaModuleProvides tcaModuleProvidesArgv 
	  tcaModuleRequires tcaModuleRequiresArgv
	  tcaConnectModule tcaIgnoreLogging tcaResumeLogging tcaClose
	  tcaRegisterNamedFormatter tcaRegisterMessage tcaPreloadMessage
	  tcaRegisterHandler tcaDeregisterHandler
	  tcaMessageRegistered tcaMessageHandlerRegistered
	  tcaRegisterGoal tcaRegisterCommand tcaRegisterInform
	  tcaRegisterQuery tcaRegisterException tcaRegisterExitProc
	  tcaRegisterResource tcaDirectResource tcaAddHndToResource
	  tcaLimitPendingResource tcaLimitPendingMessages 
	  tcaReserveResource tcaReserveModResource tcaCancelReservation
	  tcaLockResource tcaLockModResource tcaUnlockResource
	  tcaAddHndToResource tcaRegisterPointMonitor tcaRegisterDemonMonitor
	  tcaRegisterPollingMonitor tcaCreateMonitorOptions
	  tcaPointMonitor tcaPointMonitorWithConstraints
	  tcaIntervalMonitor tcaIntervalMonitorWithConstraints tcaFireDemon
	  tcaIsSameReference tcaReferenceId tcaReferenceName tcaReferenceData
	  tcaReferenceRelease tcaReferenceStatus tcaRefFree
	  tcaAddChildReference tcaCreateReference tcaFindParentReference
	  tcaFindFirstChild tcaFindLastChild tcaFindNextChild
	  tcaFindPreviousChild tcaFindFailedReference tcaFindTopLevelReference
	  tcaFindAnteReferenceByName tcaFindChildByName
	  tcaKillTaskTree tcaKillSubTaskTree tcaDisplayTaskTree
	  tcaFreeData tcaFreeReply
	  tcaStartOf tcaEndOf tcaHandlingOf tcaAchievementOf tcaPlanningOf
	  tcaTplConstrain tcaRetry tcaByPassException tcaGetExceptionInfo
	  tcaAddExceptionHandler tcaAddExceptionHandlerToMessage
	  tcaTapMessage tcaTapReference tcaFindTappedReference tcaRemoveTap
	  tcaCleanUpAfterAchieved tcaRegisterInitProc tcaRegisterVar tcaSetVar
	  tcaGetVar tcaGetSetVar tcaWatchVar tcaUnwatchVar tcaIgnoreVarLogging
	  tcaFreeVar tcaGetVersionMajor tcaGetVersionMinor 
	  tcaSetContext tcaGetContext

	  EXC_INFO_TYPE-CRef EXC_INFO_TYPE-ERef EXC_INFO_TYPE-numOfRetries 
	  make-EXC_INFO_TYPE 
	  ))

;;; Package information for allegro 4.0.1
#+(OR CLTL2 CMU)
(defpackage :TASK-CONTROL
  (:nicknames "TC" "TCA")
  (:export
   ;; Communication Return Values
   Success TimeOut Failure WrongVersion MsgUndefined
   WrongMsgClass NullReply

   *WAITFOREVER* defstruct_TCA

   ;; Export constants for message taps.
   UnknownCondition  WhenSent  BeforeHandling  WhileHandling  
   AfterHandled  AfterReplied  AfterSuccess  AfterFailure  
   BeforeAchieving  WhileAchieving 
   AfterAchieved  WhenAchieved  BeforePlanning  WhilePlanning  
   AfterPlanned WhenPlanned WhenSuccess WhenFailure  WhenKilled 

   NO_TPLCONSTR SEQ_ACH SEQ_PLANNING PLAN_FIRST DELAY_PLANNING

   tcaRegisterGoalMessage tcaRegisterCommandMessage
   tcaRegisterInformMessage tcaRegisterConstraintMessage
   tcaRegisterQueryMessage tcaRegisterExceptionMessage
   tcaRegisterBroadcastMessage tcaRegisterMultiQueryMessage
   VALID_TIME TPL_ORDER

   tcaInform tcaAddConstraint tcaQuery tcaReply tcaNullReply
   tcaEnableDistributedResponses tcaDisableDistributedResponses
   tcaQuerySend tcaQueryReceive
   tcaExpandGoal tcaExpandGoalWithConstraints
   tcaExecuteCommand tcaExecuteCommandWithConstraints
   tcaWaitForCommand tcaWaitForCommandWithConstraints
   tcaExecute tcaExecuteWithConstraints 
   tcaExecuteCommandAsync tcaExecuteAsync
   tcaMaybeExecute tcaMaybeExecuteWithConstraints tcaMaybeExecuteAsync
   tcaSuccess tcaFailure tcaRaiseException
   tcaBroadcast tcaMultiQuery tcaMultiReceive
   tcaServerMachine tcaRootNode tcaDefaultTime tcaGetServer
   tcaGetConnections tcaWaitUntilReady tcaModuleListen tcaHandleMessage
   tcaModuleProvides tcaModuleProvidesArgv 
   tcaModuleRequires tcaModuleRequiresArgv
   tcaConnectModule tcaIgnoreLogging tcaResumeLogging tcaClose
   tcaRegisterNamedFormatter tcaRegisterMessage tcaPreloadMessage
   tcaRegisterHandler tcaDeregisterHandler
   tcaMessageRegistered tcaMessageHandlerRegistered
   tcaRegisterGoal tcaRegisterCommand tcaRegisterInform
   tcaRegisterQuery tcaRegisterException tcaRegisterExitProc
   tcaRegisterResource tcaDirectResource tcaAddHndToResource
   tcaLimitPendingResource tcaLimitPendingMessages 
   tcaReserveResource tcaReserveModResource tcaCancelReservation
   tcaLockResource tcaLockModResource tcaUnlockResource
   tcaAddHndToResource tcaRegisterPointMonitor tcaRegisterDemonMonitor
   tcaRegisterPollingMonitor tcaCreateMonitorOptions
   tcaPointMonitor tcaPointMonitorWithConstraints
   tcaIntervalMonitor tcaIntervalMonitorWithConstraints tcaFireDemon
   tcaIsSameReference tcaReferenceId tcaReferenceName tcaReferenceData
   tcaReferenceRelease tcaReferenceStatus tcaRefFree
   tcaAddChildReference tcaCreateReference tcaFindParentReference
   tcaFindFirstChild tcaFindLastChild tcaFindNextChild
   tcaFindPreviousChild tcaFindFailedReference tcaFindTopLevelReference
   tcaFindAnteReferenceByName tcaFindChildByName
   tcaKillTaskTree tcaKillSubTaskTree tcaDisplayTaskTree
   tcaFreeData tcaFreeReply
   tcaStartOf tcaEndOf tcaHandlingOf tcaAchievementOf tcaPlanningOf
   tcaTplConstrain tcaRetry tcaByPassException tcaGetExceptionInfo
   tcaAddExceptionHandler tcaAddExceptionHandlerToMessage
   tcaTapMessage tcaTapReference tcaFindTappedReference tcaRemoveTap
   tcaCleanUpAfterAchieved tcaRegisterInitProc tcaRegisterVar tcaSetVar
   tcaGetVar tcaGetSetVar tcaWatchVar tcaUnwatchVar tcaIgnoreVarLogging
   tcaFreeVar tcaGetVersionMajor tcaGetVersionMinor 
   tcaSetContext tcaGetContext

   EXC_INFO_TYPE-CRef EXC_INFO_TYPE-ERef EXC_INFO_TYPE-numOfRetries 
   make-EXC_INFO_TYPE 
   ))

#+(OR CLTL2 CMU) (in-package "TC")

(defun parentDir (dir)
  (subseq dir 
	  0
	  (1+ (position #\/ dir
			:from-end T :end (1- (length dir))))))

(defparameter *tcaDir* nil
  "The tca root directory")

(eval-when (eval load compile) 
  (setf *tcaDir* (make-pathname :directory 
				(reverse 
				 (rest
				  (reverse
				   (pathname-directory *load-truename*))))))
  (format t "~%Loading from directory ~A" *tcaDir*)
  )

#+ALLEGRO (load ""
      :foreign-files 
;;      `(,(format nil "~A/lib/libtca_lisp.a" *tcaDir*))
      `(,(merge-pathnames (make-pathname :name "lib/libtca_lisp.a") *tcaDir*)
	;;"/usr/cs/lib/libc_p.a" 
	)
      :unreferenced-lib-names
      (list
       (ff::convert-to-lang "returnLispFlagGlobal")
       (ff::convert-to-lang "tcaRegisterResource")
       (ff::convert-to-lang "tcaRegisterPointMonitor")
       (ff::convert-to-lang "tcaIsSameReference")
       (ff::convert-to-lang "tcaRetry")
       (ff::convert-to-lang "tcaTapMessage")))

;;;;;;;;;;;;;

(load (merge-pathnames (make-pathname :directory "lisp" :name "formatters") 
		       *tcaDir*))
(load (merge-pathnames (make-pathname :directory "lisp" :name "map")
		       *tcaDir*))
(load (merge-pathnames (make-pathname :directory "lisp" :name "primFmttrs")
		       *tcaDir*))
(load (merge-pathnames (make-pathname :directory "lisp" 
				      :name "tcaForeignCalls") 
		       *tcaDir*))
