/******************************************************************************
 *
 * PROJECT: Carnegie Mellon Planetary Rover Project
 *          Task Control Architecture 
 * 
 * (c) Copyright 1991 Christopher Fedor and Reid Simmons.  All rights reserved.
 * 
 * MODULE: communications
 *
 * FILE: centralMsg.h
 *
 * ABSTRACT:
 * 
 * Central message definitions.
 *
 * $Source: /afs/cs.cmu.edu/project/TCA/Master/tcaV8/src/centralMsg.h,v $ 
 * $Revision: 1.5 $
 * $Date: 1995/07/12 04:54:20 $
 * $Author: rich $
 *
 * REVISION HISTORY:
 *
 * $Log: centralMsg.h,v $
 * Revision 1.5  1995/07/12  04:54:20  rich
 * Release of 8.0.
 * Fixed problems with sending between machines of different endien.
 *
 * Revision 1.4  1995/04/07  05:02:48  rich
 * Fixed GNUmakefiles to find the release directory.
 * Cleaned up libc.h file for sgi and vxworks.  Moved all system includes
 * into libc.h
 * Got direct queries to work.
 * Fixed problem in allocating/initializing generic mats.
 * The direct flag (-c) now mostly works.  Connect message has been extended to
 * indicate when direct connections are the default.
 * Problem with failures on sunOS machines.
 * Fixed problem where tcaError would not print out its message if logging had
 * not been initialized.
 * Fixed another memory problem in modVar.c.
 * Fixed problems found in by sgi cc compiler.  Many type problems.
 *
 * Revision 1.3  1995/03/28  01:14:20  rich
 * - Added ability to log data with direct connections.  Also fixed some
 * problems with global variables. It now uses broadcasts for watching variables.
 * - Added preliminary memory recovery routines to handle out of memory
 * conditions.  It currently purges items from resource queues.  Needs to
 * be tested.
 * - If the CENTRALHOST environment variable is not set, try the current
 * host.
 * - Fixed a problem with central registered messages that caused the parsed
 * formatters to be lost.
 * - Added const declarations where needed to the prototypes in tca.h.
 * - tcaGetConnections: Get the fd_set.  Needed for direct connections.
 * - Added tcaExecute and tcaExecuteWithConstraints.  Can "execute" a goal
 *   or command.
 * - tcaPreloadMessage: Preload the definition of a message from the
 *   central server.
 *
 * Revision 1.2  1994/05/25  17:32:29  reids
 * Added utilities to limit the number of pending messages
 *
 * Revision 1.1  1994/05/17  23:15:17  rich
 * Added global variables and associated routines.
 * Added some error checking.  The central connection is now set to -1
 * rather than zero to prevent tca messages from being send to stdout.
 * Now compiles on the sgi machines.  Still need to have the endian and
 * alignment figured out automatically.
 *
 *
 ****************************************************************/

#ifndef INCcentralMsg
#define INCcentralMsg

#define TCA_MSG_INFO_QUERY        "msgInfoMsg"
#define TCA_MSG_INFO_QUERY_FORMAT "{string, string}"
#define TCA_MSG_INFO_QUERY_REPLY  "*{int, string, int, format, format}"

#define TCA_HND_INFO_QUERY        "hndInfoMsg"
#define TCA_HND_INFO_QUERY_FORMAT TCA_MSG_INFO_QUERY_FORMAT
#define TCA_HND_INFO_QUERY_REPLY  TCA_MSG_INFO_QUERY_FORMAT

#define TCA_CLASS_INFO_QUERY        "classInfoMsg"
#define TCA_CLASS_INFO_QUERY_FORMAT "int"
#define TCA_CLASS_INFO_QUERY_REPLY  "*{int, format}"

#define TCA_PROVIDES_INFORM        "tca_moduleProvides"
#define TCA_PROVIDES_INFORM_FORMAT TCA_STR_LIST_FORMAT

#define TCA_REQUIRES_INFORM        "tca_moduleRequires"
#define TCA_REQUIRES_INFORM_FORMAT TCA_STR_LIST_FORMAT

#define TCA_CONNECT_QUERY        "newModuleConnectMsg"
#define TCA_CONNECT_QUERY_FORMAT "{string, string}"
#define TCA_CONNECT_QUERY_REPLY  "{{int, int},boolean}"

#define TCA_REGISTER_MSG_INFORM        "registerMessageMsg"
#define TCA_REGISTER_MSG_INFORM_FORMAT "{string, int, string, string}"

#define TCA_REGISTER_HND_INFORM        "registerHandlerMsg"
#define TCA_REGISTER_HND_INFORM_FORMAT "{int, string, string}"

#define TCA_DEREGISTER_HND_INFORM        "deregisterHandlerMsg"
#define TCA_DEREGISTER_HND_INFORM_FORMAT "{int, string, string}"

#define TCA_NAMED_FORM_INFORM        "registerNamedFormMsg"
#define TCA_NAMED_FORM_INFORM_FORMAT "{string, string}"

#define TCA_WAIT_QUERY        "waitMsg"
#define TCA_WAIT_QUERY_FORMAT NULL
#define TCA_WAIT_QUERY_REPLY  "int"

#define TCA_IGNORE_LOGGING_INFORM        "ignoreLogging"
#define TCA_IGNORE_LOGGING_INFORM_FORMAT "string"

#define TCA_RESUME_LOGGING_INFORM        "resumeLogging"
#define TCA_RESUME_LOGGING_INFORM_FORMAT "string"

#define TCA_CLOSE_INFORM        "tcaCloseMsg"
#define TCA_CLOSE_INFORM_FORMAT NULL

#define TCA_DIRECT_RES_INFORM        "directResMsg"
#define TCA_DIRECT_RES_INFORM_FORMAT "{int, string}"

#define TCA_DIRECT_INFO_QUERY        "directInfoMsg"
#define TCA_DIRECT_INFO_QUERY_FORMAT "string"
#define TCA_DIRECT_INFO_QUERY_REPLY  "{int, int, int, string, string}"

#define TCA_MAP_NAMED         "map"
#define TCA_MAP_NAMED_FORMAT  "fmat"

#define TCA_REGISTER_RESOURCE_INFORM        "registerResourceMsg"
#define TCA_REGISTER_RESOURCE_INFORM_FORMAT "{int, string}"

#define TCA_HANDLER_TO_RESOURCE_INFORM        "addHndToResourceMsg"
#define TCA_HANDLER_TO_RESOURCE_INFORM_FORMAT "{string, string}"

#define TCA_LIMIT_PENDING_INFORM        "limitPendingMsg"
#define TCA_LIMIT_PENDING_INFORM_FORMAT "{string, string, int}"

#define TCA_RESERVE_RESOURCE_QUERY        "reserveResourceMsg"
#define TCA_RESERVE_RESOURCE_QUERY_FORMAT "string"
#define TCA_RESERVE_RESOURCE_QUERY_REPLY  "int"

#define TCA_RESERVE_MOD_RESOURCE_QUERY        "reserveModResourceMsg"
#define TCA_RESERVE_MOD_RESOURCE_QUERY_FORMAT "{string,string}"
#define TCA_RESERVE_MOD_RESOURCE_QUERY_REPLY  "int"

#define TCA_CANCEL_RESOURCE_INFORM        "cancelReservationMsg"
#define TCA_CANCEL_RESOURCE_INFORM_FORMAT "int"

#define TCA_LOCK_RESOURCE_QUERY        "lockResourceMsg"
#define TCA_LOCK_RESOURCE_QUERY_FORMAT "string"
#define TCA_LOCK_RESOURCE_QUERY_REPLY  "int"

#define TCA_LOCK_MOD_RESOURCE_QUERY        "lockModResourceMsg"
#define TCA_LOCK_MOD_RESOURCE_QUERY_FORMAT "{string, string}"
#define TCA_LOCK_MOD_RESOURCE_QUERY_REPLY  "int"

#define TCA_UNLOCK_RESOURCE_INFORM        "unlockResourceMsg"
#define TCA_UNLOCK_RESOURCE_INFORM_FORMAT "int"

#define TCA_REG_MONITOR_INFORM        "regMonitorMsg"
#define TCA_REG_MONITOR_INFORM_FORMAT "{int, string, string, string, string}"

#define TCA_DELAY_CMD        "tcaDelayCommand"
#define TCA_DELAY_CMD_FORMAT "int"

#define TAP_REF_INFORM_FORM "{int, TCA_REF_PTR, string}"

#define TAP_MSG_INFORM_FORM "{int, string, string}"

#define TCA_CONTRAINT_INFORM        "tplConstrain"
#define TCA_CONTRAINT_INFORM_FORMAT "{TimePoint, string, TimePoint}"

#define TCA_REGISTER_VAR_INFORM        "registerGlobalVar"
#define TCA_REGISTER_VAR_INFORM_FORMAT "{string,string}"

/* Task Tree Messages */

#define TCA_CREATE_REF_QUERY        "createRefMsg"
#define TCA_CREATE_REF_QUERY_FORMAT "int"
#define TCA_CREATE_REF_QUERY_REPLY  "int"

#define TCA_ADD_CHILD_QUERY        "addChildRef"
#define TCA_ADD_CHILD_QUERY_FORMAT "{int, int}"
#define TCA_ADD_CHILD_QUERY_REPLY  "int"

#define TCA_FIND_PARENT_QUERY        "findParentRef"
#define TCA_FIND_PARENT_QUERY_FORMAT "int"
#define TCA_FIND_PARENT_QUERY_REPLY  "TCA_REF_PTR"

#define TCA_FIRST_CHILD_QUERY        "findFirstChild"
#define TCA_FIRST_CHILD_QUERY_FORMAT "int"
#define TCA_FIRST_CHILD_QUERY_REPLY  "TCA_REF_PTR"

#define TCA_LAST_CHILD_QUERY        "findLastChild"
#define TCA_LAST_CHILD_QUERY_FORMAT "int"
#define TCA_LAST_CHILD_QUERY_REPLY  "TCA_REF_PTR"

#define TCA_NEXT_CHILD_QUERY        "findNextChild"
#define TCA_NEXT_CHILD_QUERY_FORMAT "int"
#define TCA_NEXT_CHILD_QUERY_REPLY  "TCA_REF_PTR"

#define TCA_PREV_CHILD_QUERY        "findPrevChild"
#define TCA_PREV_CHILD_QUERY_FORMAT "int"
#define TCA_PREV_CHILD_QUERY_REPLY  "TCA_REF_PTR"

#define TCA_FAILED_CHILD_QUERY        "findFailedRef"
#define TCA_FAILED_CHILD_QUERY_FORMAT "int"
#define TCA_FAILED_CHILD_QUERY_REPLY  "TCA_REF_PTR"

#define TCA_FIND_TOPLEVEL_QUERY        "findTopLevelRef"
#define TCA_FIND_TOPLEVEL_QUERY_FORMAT "int"
#define TCA_FIND_TOPLEVEL_QUERY_REPLY  "TCA_REF_PTR"

#define TCA_ANTE_BY_NAME_QUERY        "findAnteRefByName"
#define TCA_ANTE_BY_NAME_QUERY_FORMAT "{int, string}"
#define TCA_ANTE_BY_NAME_QUERY_REPLY  "TCA_REF_PTR"

#define TCA_CHILD_BY_NAME_QUERY        "findChildByName"
#define TCA_CHILD_BY_NAME_QUERY_FORMAT "{int, string}"
#define TCA_CHILD_BY_NAME_QUERY_REPLY  "TCA_REF_PTR"

#define TCA_KILL_TREE_INFORM        "killTaskTree"
#define TCA_KILL_TREE_INFORM_FORMAT "int"

#define TCA_KILL_SUBTREE_INFORM        "killSubTaskTree"
#define TCA_KILL_SUBTREE_INFORM_FORMAT "int"

#define TCA_DISPLAY_TREE_INFORM        "displayTaskTree"
#define TCA_DISPLAY_TREE_INFORM_FORMAT "int"

#define TCA_REF_DATA_QUERY        "referenceData"
#define TCA_REF_DATA_QUERY_FORMAT "int"
#define TCA_REF_DATA_QUERY_REPLY  NULL

#define TCA_REF_RELEASE_INFORM        "referenceRelease"
#define TCA_REF_RELEASE_INFORM_FORMAT "int"

#define TCA_REF_STATUS_QUERY        "referenceStatusMsg"
#define TCA_REF_STATUS_QUERY_FORMAT "int"
#define TCA_REF_STATUS_QUERY_REPLY  "int"

/* Define constants for central varibles. */

#define TCA_TERMINAL_LOG_VAR  "CENTRAL_TERMINAL_LOG_VAR"
#define TCA_FILE_LOG_VAR  "CENTRAL_FILE_LOG_VAR"
/*#define TCA_LOG_VAR_FORMAT "{FILE,int,int,int,int,int,int,int,int,int,int,int}"*/
#define TCA_LOG_VAR_FORMAT "{*int,int,int,int,int,int,int,int,int,int,int,int}"

#define TCA_CENTRAL_MODULE_DATA_VAR "CENTRAL_MOD_VAR"

#define TCA_MOD_DATA_NAME "TCA_MOD_DATA"
#define TCA_MOD_DATA_FORMAT "{string,string}"

#define TCA_LIST_NAME "TCA_LIST"
#define TCA_LIST_FORMAT "*int"

#define TCA_RESOURCE_STATUS_NAME "TCA_RESOURCE_STATUS"
#define TCA_RESOURCE_STATUS_FORMAT "int"
#define TCA_RESOURCE_FORMAT "int,string,int,*int,RESOURCE_STATUS,TCA_LIST,TCA_LIST,int,LIST_PTR"
#define TCA_MODULE_DATA_VAR_FORMAT "int,int,int,int,*MOD_DATA_TYPE,*_RESOURCE,*_LIST,*_LIST"

#endif /* INCcentralMsg */
