
/*
 * File: MsgData.h
 * Author: Domingo Gallardo, CMU
 * Purpose: Public data referred to TCA messages
 * 
 *
 * REVISION HISTORY
 *
 * $Log: MsgData.h,v $
 * Revision 1.4  1995/05/31  20:58:34  rich
 * Fixed conflict with tca declarations.
 *
 * Revision 1.3  1995/04/19  14:31:58  rich
 * Added int32 for dealing with tca identifiers.
 *
 * Revision 1.2  1995/04/07  05:06:57  rich
 * Fixed GNUmakefiles to find the release directory.
 * Fixed problems found by sgi cc compiler.  It would not compile.
 *
 * Revision 1.1  1995/04/05  18:31:13  rich
 * Moved tview files to a subdirectory.
 *
 * Revision 1.1  1994/05/31  03:26:14  rich
 * Moved Domingo's tview tool into the main tca module.
 *
 * Revision 1.3  1994/05/27  05:33:20  rich
 * Can now read from file that is being written to.
 * Fixed Menu and button names.
 * Fixed Indentation.
 * Added menu item to change orientation of the tree.
 *
 * Revision 1.2  1993/08/13  01:43:06  domingo
 * Updated function declarations for compilation under gcc in ANSI C (still
 * a lot of warnings to fix).
 * Added automatic logging.
 *
 *
 * Jan 1 1992 - Domingo Gallardo at School of Computer Science, CMU
 * Changed the type of all the strings in the data structure; they are not
 * initialized strings but void pointers.
 *
 * Dec 25 1992 - Domingo Gallardo at School of Computer Science, CMU
 * Created.
 *
 */

#ifndef _MSGDATA_H
#define _MSGDATA_H

#include <X11/Intrinsic.h>

typedef enum {
  STATUS_NULL, SENT, RESUMPTION, PENDING, 
  INACTIVE, TOP_LEVEL_QUERY, COMPLETED, WAIT_FOR_REPLAY}  msgStatus;

typedef enum {
  TYPE_NULL, QUERY, GOAL, COMMAND, INFORM, EXCEPTION,
  KILLED, INTERVAL_MONITOR, POLLING_MONITOR,
  FIRE_DEMON, REPLY, FAILURE, SUCCESS, BYPASS,
  POINT_CONSTRAINT, RETRY, WILL_KILL, TEMP_CONSTRAINT, WARNING} msgType;

typedef enum {
  NULL_KILL_SITUATION, WHEN_REFERENCES_RELEASED, WHEN_COMPLETES,
  WHEN_SUBMESSAGES_COMPLETES} killSituation;

typedef enum {
  MSG_SEQ_ACH, MSG_SEQ_PLAN, MSG_PLAN_FIRST, MSG_DELAY_PLAN,
  MSG_INV_SEQ_ACH, MSG_INV_SEQ_PLAN} temporalConstraint;

typedef enum {
  START, END
} intervalOp;


typedef enum {
  HANDLING_INTERVAL, PLANNING_INTERVAL, ACHIVEMENT_INTERVAL
} interval;


typedef struct _MsgData {
  msgType      type;
  String       name;
  int32        id;
  int32        parent_id;
  String       source;
  String       dest;
  msgStatus    status;
  int          time;
  int          hours;
  int          minutes;
  int          seconds;
  int          m_seconds;
  killSituation kill_when;
  String       msg1;
  String       msg2;
  int32        id1; 
  int32        id2;
  int32        temporal_constraint;
  interval     interval1;
  interval     interval2;
  intervalOp   point1;
  intervalOp   point2;
  char         point_TC;  /*     CONSTRAINT        CHAR
			   *
			   *         <               <
			   *         =               =
			   *         >               >
			   *         <=              /
			   *         >=              \
			   */
  
} _MsgData, *MsgData;

#endif
