/*****************************************************************************
 * PROJECT: TCA
 *
 * (c) Copyright Richard Goodwin, 1995. All rights reserved.
 *
 * FILE: tcaDev.h
 *
 * ABSTRACT:
 * 
 * This file provides routines for connecting TCA to the set of device
 * interfaces (see devUtils.{c,h}).
 *
 * ADAPTED FROM XAVIER SOFTWARE TO FACILITATE ADDING I/O TO MODULES.
 *
 * $Source: /afs/cs.cmu.edu/project/TCA/Master/tcaV8/utils/tcaDev.h,v $ 
 * $Revision: 1.6 $
 * $Date: 1995/08/05 23:23:38 $
 * $Author: rich $
 *
 * REVISION HISTORY:
 *
 * $Log: tcaDev.h,v $
 * Revision 1.6  1995/08/05  23:23:38  rich
 * Added functional interface to devUtils.  See the README for details.
 *
 * Revision 1.5  1995/07/19  14:32:58  rich
 * Error checking on select and accept calls.
 *
 * Revision 1.4  1995/07/06  21:23:36  rich
 * Solaris and Linux changes.
 *
 * Revision 1.3  1995/04/07  05:11:22  rich
 * Fixed GNUmakefiles to find the release directory.
 * Moved all system includes into libc.h
 * Moved global variables into the c files and got rid of #define DECLARE_...
 * Now works with xavier stuff.
 *
 * Revision 1.2  1995/04/04  19:48:27  rich
 * Added sgi support.
 * Changed setAlarm to setTimer.
 * Numerous improvements and bug fixes in devUtils.
 *
 * Revision 1.1  1995/03/30  15:53:35  rich
 * DBMALLOC works.  To use "gmake -k -w DBMALLOC=DBMALLOC install"
 * Added ezx library and the devutils library.
 *
 *
 *****************************************************************************/

#ifndef TCA_DEV_LOADED
#define TCA_DEV_LOADED

#include "tca/libc.h"
#include "tca.h"
#include "tca/devUtils.h"

/* Utility routines */

void TCA_outputHnd(int fd, long chars_available);

void TCA_connect(char *moduleName,void registerAll(void),
		 void disconnectFn(void),void reconnectFn(void),
		 const char **provides, const char **requires);

void TCA_disconnect(DEV_PTR ignore);

BOOLEAN TCA_isConnected(void);

BOOLEAN TCA_isDisconnected(void);

typedef struct {
  DEV_PTR dev;
  char *moduleName;
  void (* registerAll)(void);
  void (* disconnectHnd)(void);
  void (* reconnectHnd)(void);
  const char **provides;
  const char **requires;
}  TCA_TYPE, *TCA_PTR;


extern TCA_PTR    TCA_device;

#endif
