/*****************************************************************************
 * PROJECT: TCA
 *
 * (c) Copyright Richard Goodwin, 1995. All rights reserved.
 *
 * FILE: x11Utils.c
 *
 * ABSTRACT:
 * 
 * This file provides routines having the X11 main loop handle tca events.
 *
 * ADAPTED FROM XAVIER SOFTWARE TO FACILITATE ADDING I/O TO MODULES.
 *
 * $Source: /afs/cs.cmu.edu/project/TCA/Master/tcaV8/utils/x11Utils.c,v $ 
 * $Revision: 1.2 $
 * $Date: 1995/06/14 03:26:22 $
 * $Author: rich $
 *
 * REVISION HISTORY:
 *
 * $Log: x11Utils.c,v $
 * Revision 1.2  1995/06/14  03:26:22  rich
 * Added DBMALLOC_DIR.
 * Fixed problems with multiple connections.
 *
 * Revision 1.1  1995/04/07  05:16:39  rich
 * Added x11Utils to connect tca to the X11 main loop.
 * Added sim device to use in creating simulators.
 *
 *
 *****************************************************************************/

#include "tca/libc.h"

#include <X11/Xfuncproto.h>
#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/Xlib.h>

#include <X11/Xatom.h>
#include <X11/IntrinsicP.h>
#include <X11/Core.h>
#include <X11/CoreP.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>

#include <X11/Xaw/Label.h>
#include <X11/Xaw/Viewport.h>

#include <X11/Shell.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Dialog.h>

#include "tca.h"
#include "devUtils.h"

/*****************************************************************************
 * Global constants 
 *****************************************************************************/

/*****************************************************************************
 * Global variables 
 *****************************************************************************/

static XtAppContext Keep_Context = NULL;
static XtInputId X_Inputs[FD_SETSIZE];

/*****************************************************************************
 *
 * FUNCTION: void X11_defaultOutputHnd(int fd, long chars_available)
 *
 * DESCRIPTION: Handles X11 events.
 *
 * INPUTS:
 *
 * OUTPUTS: 
 *
 * HISTORY:
 *
 *****************************************************************************/
static void tcaCallbackProc(XtPointer closure, int* fd, XtInputId* id);

static void updateXInputs(void)
{
  static fd_set X11_TCA = NO_FDS;
  int i;
  fd_set *tcaFDS;

  tcaFDS = tcaGetConnections();
  
  for(i=0;i<FD_SETSIZE; i++) {
    if (FD_ISSET(i,&X11_TCA) && !FD_ISSET(i,tcaFDS)) {
      /* Remove connection that is no longer valid. */
      if (X_Inputs[i] != 0) {
	XtRemoveInput(X_Inputs[i]);
	X_Inputs[i] = 0;
      }
      FD_CLR(i,&X11_TCA);
    } else if (!FD_ISSET(i,&X11_TCA) && FD_ISSET(i,tcaFDS)) {
      /* Add a new connection. */
      XtAppAddInput(Keep_Context, i, 
		    (XtPointer)(XtInputReadMask | XtInputExceptMask),
		    tcaCallbackProc, NULL);
    }
  }

}

static void tcaCallbackProc(XtPointer closure, int* fd, XtInputId*	id)
{
  tcaHandleMessage(0);
  updateXInputs();
}

void connectTCA_X11(XtAppContext app_context)
{
  int i;

  Keep_Context = app_context;
  for(i=0;i<FD_SETSIZE; i++) {
    X_Inputs[i] = 0;
  }
  updateXInputs();
}
